/**************************************************************************//**
 * @item     CosyOS-II 行为同步测试
 * @file     test_xwtb_ressus.c
 * @brief    恢复挂起测试
 * @detail   通过挂起自身/恢复，实现中断对任务、任务对任务的行为同步（事件同步）。
 * @service  uSuspendTasking;   挂起自身任务
             uResumeTask(task); 任务中恢复任务
             iResumeTask(task); 中断中恢复任务
 * @author   迟凯峰
 * @version  V1.0.0
 * @date     2024.04.30
 ******************************************************************************/

#include "..\test_xwtb.h"
#if TEST_XWTB_OBJECT == TEST_XWTB_RESSUS

#include "..\System\os_link.h"
#include "typedef.h"
#include <stdio.h>

/**
 * \brief    task_3
 * \detail   处理更为耗时的事务。
 */
uCreateTask(task_3, 3, 64, 0, 0)
{
	if(1){
		static u16 i = 0;
		void led_task_3(void);
		P03 = 0; // 任务被同步运行后，需立即关闭测量信号
		printf("task_2 -> task_3: %u\r\n", ++i); // 打印 task_2 同步至 task_3 的次数
		led_task_3();
	}
	uSuspendTasking;
	uEndTasking;
}

/**
 * \brief    task_2
 * \detail   先对事件做必要的紧急处理，再同步至 task_1 或 task_3 做后续处理。
 */
uCreateTask(task_2, 2, 64, 0, 0)
{
	uSuspendTasking;
	if(1){
		static u16 i = 0;
		void led_task_2(void);
		P02 = 0; // 任务被同步运行后，需立即关闭测量信号
		printf("TIME_3 -> task_2: %u\r\n", ++i); // 打印 TIME_3 同步至 task_2 的次数
		led_task_2();
		
		P04 = ~P04;
		if(P04){
			P03 = 1; // 先开启测量信号，再同步
			uResumeTask(task_3); /*!< 同步至 task_3，可以立即切换至 task_3 */
		}
		else{
			uExternTask(task_1);
			P01 = 1; // 先开启测量信号，再同步
			uResumeTask(task_1); /*!< 同步至 task_1，无法立即切换至 task_1 */
		}
	}
	uEndTasking;
}

/**
 * \brief    task_1
 * \detail   处理更为耗时的事务。
 */
uCreateTask(task_1, 1, 64, 0, 0)
{
	if(1){
		static u16 i = 0;
		void led_task_1(void);
		P01 = 0; // 任务被同步运行后，需立即关闭测量信号
		printf("task_2 -> task_1: %u\r\n", ++i); // 打印 task_2 同步至 task_1 的次数
		led_task_1();
	}
	uSuspendTasking;
	uEndTasking;
}

/**
 * \brief    定时器3中断
 * \detail   先对事件做必要的紧急处理，再同步至 task_2 做后续处理。
 */
void TIME_3(void) interrupt 19 MCUCFG_USING
{
	AUXINTIF &= ~T3IF;
	P02 = 1; // 先开启测量信号，再同步
	iResumeTask(task_2); /*!< 同步至 task_2 */
}

/**
 * \brief    启动钩子
 * \detail   在启动任务中被调用，可在此启动用户任务。
 */
void start_hook(void)
{
	P01 = 0;               /*!< task_2 同步至 task_1，任务切换时间测量 */
	P02 = 0;               /*!< TIME_3 同步至 task_2，中断响应时间测量 */
	P03 = 0;               /*!< task_2 同步至 task_3，任务切换时间测量 */
	uStartTask(task_3, 1); /*!< 启动 task_3，初始状态为挂起 */
	uStartTask(task_2, 0); /*!< 启动 task_2，初始状态为就绪 */
	uStartTask(task_1, 1); /*!< 启动 task_1，初始状态为挂起 */
	/* 初始化定时器3 */
	T4T3M &= 0xF0;         // 停止计数, 定时模式, 12T模式, 不输出时钟
	T3L = (u8)((65536 - SYSCFG_SYSCLK / 12 / 40));
	T3H = (u8)((65536 - SYSCFG_SYSCLK / 12 / 40) >> 8);
	TM3PS = 99;
	T4T3M |= 0x08;
	IE2 |= ET3;
}

#endif
